# Default Configuration for your project

DIRECTORY = "stark"

PLUGINS = f"{DIRECTORY}/plugins"

BOT_PLUGINS = f"{DIRECTORY}/assistant"

LOCALIZATION = f"{DIRECTORY}/localization"

SET_BOT_MENU = True

CMD_PREFIXES = ["."]

BOT_CMD_PREFIXES = ["/"]

START = "Thank you for starting this bot."

HELP = """
Available Commands are:

{commands}
"""

ABOUT = "About This Bot \nDeveloped using @PyStark"

TIMEZONE = "Asia/Kolkata"

START_IN_GROUPS = "Hey :) Ping me privately if you have any questions on how to use me."
